S(N_("Success"))
S(N_("Probable success"))
S(N_("Not found"))
S(N_("Probably not found"))
S(N_("Cache expired"))
S(N_("NIS+ servers unreachable"))
S(N_("Unknown object"))
S(N_("Server busy, try again"))
S(N_("Generic system error"))
S(N_("First/next chain broken"))
S(N_("Permission denied"))
S(N_("Not owner"))
S(N_("Name not served by this server"))
S(N_("Server out of memory"))
S(N_("Object with same name exists"))
S(N_("Not master server for this domain"))
S(N_("Invalid object for operation"))
S(N_("Malformed name, or illegal name"))
S(N_("Unable to create callback"))
S(N_("Results sent to callback proc"))
S(N_("Not found, no such name"))
S(N_("Name/entry isn't unique"))
S(N_("Modification failed"))
S(N_("Database for table does not exist"))
S(N_("Entry/table type mismatch"))
S(N_("Link points to illegal name"))
S(N_("Partial success"))
S(N_("Too many attributes"))
S(N_("Error in RPC subsystem"))
S(N_("Missing or malformed attribute"))
S(N_("Named object is not searchable"))
S(N_("Error while talking to callback proc"))
S(N_("Non NIS+ namespace encountered"))
S(N_("Illegal object type for operation"))
S(N_("Passed object is not the same object on server"))
S(N_("Modify operation failed"))
S(N_("Query illegal for named table"))
S(N_("Attempt to remove a non-empty table"))
S(N_("Error in accessing NIS+ cold start file.  Is NIS+ installed?"))
S(N_("Full resync required for directory"))
S(N_("NIS+ operation failed"))
S(N_("NIS+ service is unavailable or not installed"))
S(N_("Yes, 42 is the meaning of life"))
S(N_("Unable to authenticate NIS+ server"))
S(N_("Unable to authenticate NIS+ client"))
S(N_("No file space on server"))
S(N_("Unable to create process on server"))
S(N_("Master server busy, full dump rescheduled."))
