#include <sunrpc/rpc/xdr.h>

libc_hidden_proto (xdrstdio_create)
libc_hidden_proto (xdr_array)
libc_hidden_proto (xdr_bool)
libc_hidden_proto (xdr_bytes)
libc_hidden_proto (xdr_char)
libc_hidden_proto (xdr_double)
libc_hidden_proto (xdr_enum)
libc_hidden_proto (xdr_float)
libc_hidden_proto (xdr_free)
libc_hidden_proto (xdr_int)
libc_hidden_proto (xdr_long)
libc_hidden_proto (xdr_netobj)
libc_hidden_proto (xdr_opaque)
libc_hidden_proto (xdr_pointer)
libc_hidden_proto (xdr_reference)
libc_hidden_proto (xdr_short)
libc_hidden_proto (xdr_string)
libc_hidden_proto (xdr_u_char)
libc_hidden_proto (xdr_u_int)
libc_hidden_proto (xdr_u_long)
libc_hidden_proto (xdr_u_short)
libc_hidden_proto (xdr_union)
libc_hidden_proto (xdr_vector)
libc_hidden_proto (xdr_void)
libc_hidden_proto (xdr_wrapstring)
libc_hidden_proto (xdrmem_create)
libc_hidden_proto (xdrrec_create)
libc_hidden_proto (xdrrec_endofrecord)
libc_hidden_proto (xdrrec_eof)
libc_hidden_proto (xdrrec_skiprecord)
libc_hidden_proto (xdr_int16_t)
libc_hidden_proto (xdr_int32_t)
libc_hidden_proto (xdr_int8_t)
libc_hidden_proto (xdr_sizeof)
libc_hidden_proto (xdr_uint16_t)
libc_hidden_proto (xdr_uint32_t)
libc_hidden_proto (xdr_uint8_t)
libc_hidden_proto (xdr_hyper)
libc_hidden_proto (xdr_u_hyper)
libc_hidden_proto (xdr_longlong_t)
libc_hidden_proto (xdr_u_longlong_t)
libc_hidden_proto (xdr_int64_t)
libc_hidden_proto (xdr_uint64_t)
libc_hidden_proto (xdr_quad_t)
libc_hidden_proto (xdr_u_quad_t)
