#undef	B0
#undef	B50
#undef	B75
#undef	B110
#undef	B134
#undef	B150
#undef	B200
#undef	B300
#undef	B600
#undef	B1200
#undef	B1800
#undef	B2400
#undef	B4800
#undef	B9600
#undef	B19200
#undef	B38400
#undef	EXTA
#undef	EXTB
#undef	ECHO
#undef	TOSTOP
#undef	NOFLSH
#undef	MDMBUF
#undef	FLUSHO
#undef	PENDIN
#undef	CERASE
#undef	CKILL
#undef	CINTR
#undef	CQUIT
#undef	CSTART
#undef	CSTOP
#undef	CEOF
#undef	CEOT
#undef	CBRK
#undef	CSUSP
#undef	CDSUSP
#undef	CRPRNT
#undef	CFLUSH
#undef	CWERASE
#undef	CLNEXT
#undef	CSTATUS
#undef	CERASE
#undef	CKILL
#undef	CINTR
#undef	CQUIT
#undef	CSTART
#undef	CSTOP
#undef	CEOF
#undef	CEOT
#undef	CBRK
#undef	CSUSP
#undef	CDSUSP
#undef	CRPRNT
#undef	CFLUSH
#undef	CWERASE
#undef	CLNEXT
#undef	CSTATUS
#undef	CERASE
#undef	CKILL
#undef	CINTR
#undef	CQUIT
#undef	CSTART
#undef	CSTOP
#undef	CEOF
#undef	CEOT
#undef	CBRK
#undef	CSUSP
#undef	CDSUSP
#undef	CRPRNT
#undef	CFLUSH
#undef	CWERASE
#undef	CLNEXT
#undef	CSTATUS
#undef	CERASE
#undef	CKILL
#undef	CINTR
#undef	CQUIT
#undef	CSTART
#undef	CSTOP
#undef	CEOF
#undef	CEOT
#undef	CBRK
#undef	CSUSP
#undef	CDSUSP
#undef	CRPRNT
#undef	CFLUSH
#undef	CWERASE
#undef	CLNEXT
#undef	CSTATUS
#undef	CERASE
#undef	CKILL
#undef	CINTR
#undef	CQUIT
#undef	CSTART
#undef	CSTOP
#undef	CEOF
#undef	CEOT
#undef	CBRK
#undef	CSUSP
#undef	CDSUSP
#undef	CRPRNT
#undef	CFLUSH
#undef	CWERASE
#undef	CLNEXT
#undef	CSTATUS
#undef	CERASE
#undef	CKILL
#undef	CINTR
#undef	CQUIT
#undef	CSTART
#undef	CSTOP
#undef	CEOF
#undef	CEOT
#undef	CBRK
#undef	CSUSP
#undef	CDSUSP
#undef	CRPRNT
#undef	CFLUSH
#undef	CWERASE
#undef	CLNEXT
#undef	CSTATUS
#undef	CERASE
#undef	CKILL
#undef	CINTR
#undef	CQUIT
#undef	CSTART
#undef	CSTOP
#undef	CEOF
#undef	CEOT
#undef	CBRK
#undef	CSUSP
#undef	CDSUSP
#undef	CRPRNT
#undef	CFLUSH
#undef	CWERASE
#undef	CLNEXT
#undef	CSTATUS
#undef	CERASE
#undef	CKILL
#undef	CINTR
#undef	CQUIT
#undef	CSTART
#undef	CSTOP
#undef	CEOF
#undef	CEOT
#undef	CBRK
#undef	CSUSP
#undef	CDSUSP
#undef	CRPRNT
#undef	CFLUSH
#undef	CWERASE
#undef	CLNEXT
#undef	CSTATUS
#undef	CERASE
#undef	CKILL
#undef	CINTR
#undef	CQUIT
#undef	CSTART
#undef	CSTOP
#undef	CEOF
#undef	CEOT
#undef	CBRK
#undef	CSUSP
#undef	CDSUSP
#undef	CRPRNT
#undef	CFLUSH
#undef	CWERASE
#undef	CLNEXT
#undef	CSTATUS
#undef	CERASE
#undef	CKILL
#undef	CINTR
#undef	CQUIT
#undef	CSTART
#undef	CSTOP
#undef	CEOF
#undef	CEOT
#undef	CBRK
#undef	CSUSP
#undef	CDSUSP
#undef	CRPRNT
#undef	CFLUSH
#undef	CWERASE
#undef	CLNEXT
#undef	CSTATUS
#undef	CERASE
#undef	CKILL
#undef	CINTR
#undef	CQUIT
#undef	CSTART
#undef	CSTOP
#undef	CEOF
#undef	CEOT
#undef	CBRK
#undef	CSUSP
#undef	CDSUSP
#undef	CRPRNT
#undef	CFLUSH
#undef	CWERASE
#undef	CLNEXT
#undef	CSTATUS
#undef	CERASE
#undef	CKILL
#undef	CINTR
#undef	CQUIT
#undef	CSTART
#undef	CSTOP
#undef	CEOF
#undef	CEOT
#undef	CBRK
#undef	CSUSP
#undef	CDSUSP
#undef	CRPRNT
#undef	CFLUSH
#undef	CWERASE
#undef	CLNEXT
#undef	CSTATUS

#include <sys/ioctl.h>
